/*
 * Copyright (C) 1996-2003 LAAS/CNRS 
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *    - Redistributions of source code must retain the above copyright
 *      notice, this list of conditions and the following disclaimer.
 *    - Redistributions in binary form must reproduce the above
 *      copyright notice, this list of conditions and the following
 *      disclaimer in the documentation and/or other materials provided
 *      with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * COPYRIGHT HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
 * ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 */
#include "config.h"

#include <signal.h>
#include <sys/time.h>
#include <stdio.h>
#include <errno.h>

#include "watchDog.h"

#ifndef OK
#  define OK 0
#endif
#ifndef ERROR
#  define ERROR (-1)
#endif

/***
 *** Watch Dogs pour UNIX
 ***/

static RETSIGTYPE watchDogHandler(int sig);

/**
 ** Variables statiques du module 
 **/
static void (*watchDogFunction)(void *);
static void *watchDogArg;

/*----------------------------------------------------------------------*/

/**
 ** watchDogArm - Arme un watch dog 
 **
 ** Il se declenchera dans delay milli-secondes
 **/

int
watchDogArm(unsigned long delay, void (*function)(void *arg), void *arg)
{
    sigset_t sigset;
    struct sigaction act;
    struct itimerval timer;

    /* Remplir la structure du timer */
    timer.it_interval.tv_sec = 0;
    timer.it_interval.tv_usec = 0;
    timer.it_value.tv_sec =  delay / 1000;
    timer.it_value.tv_usec =  (delay % 1000) * 1000;
    
    /* Bloquer SIGALRM - il est debloque' plus tard */
    sigemptyset(&sigset);
    sigaddset(&sigset, SIGALRM);
    sigprocmask(SIG_BLOCK, &sigset, NULL);

    /* Installer le handler */
    act.sa_handler = watchDogHandler;
    sigemptyset(&act.sa_mask);
    act.sa_flags = 0;	    
    sigaction(SIGALRM, &act, NULL);
    
    /* Enregistrer la fonction et son argument */
    watchDogFunction = function;
    watchDogArg = arg;

    /* Debloquer SIGALRM */
    sigprocmask(SIG_UNBLOCK, &sigset, NULL);
    
    /* Demarrer le timer */
    if (setitimer(ITIMER_REAL, &timer, NULL) < 0) {
        return(ERROR);
    }
    errno = 0;
    return(OK);
}

/*----------------------------------------------------------------------*/

/**
 ** watchDogRelease - desarme le watchDog
 **/

void
watchDogRelease(void)
{
    struct itimerval timer;
    int oerrno;

    oerrno = errno;
    timer.it_value.tv_sec = 0;
    timer.it_value.tv_usec = 0;
    timer.it_interval.tv_sec = 0;
    timer.it_interval.tv_usec = 0;
    
    setitimer(ITIMER_REAL, &timer, NULL);
    errno = oerrno;
}

/*----------------------------------------------------------------------*/

/**
 ** watchDogHandler - fonction de traitement de l'IT du watchDog
 **/
static 
RETSIGTYPE watchDogHandler(int sig)
{
    if (watchDogFunction != NULL) {
	watchDogFunction(watchDogArg);
    }
}

/*----------------------------------------------------------------------*/

