# 
# Copyright (C) 1996-2001 LAAS/CNRS 
# All rights reserved.
# 
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
# 
#    - Redistributions of source code must retain the above copyright
#      notice, this list of conditions and the following disclaimer.
#    - Redistributions in binary form must reproduce the above
#      copyright notice, this list of conditions and the following
#      disclaimer in the documentation and/or other materials provided
#      with the distribution.
# 
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
# FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
# COPYRIGHT HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
# INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
# BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
# CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
# LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
# ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.
# 
# $LAAS$
#----------------------------------------------------------------------
# Dessin du satellite PROBA
#
# Le satellite evolue selon une trajectoire orbitale circulaire dans un 
# plan orbital fixe (relativement aux etoiles). Ce plan correspond au 
# plan equatorial qui serait pivot autour d'un axe horizontal
# avec un angle "incid" (constant).
# La position du satellite sur son cercle est alors entierement definie 
# par l'angle "alpha" relativement au point sur l'equateur.
#
# L'orientation du satellite est finalement caractris par le triplet
# (roll pitch yaw) (roulie, tangage et lacet)
#
# Parametres (en degre pour les angles)
#     angleTerre : angle de rotation de la Terre (permet de positionner le
#                  satellite par rapport a la Terre sans faire tourner cette
#                  derniere. Mettre 0 sinon.)
#     incid : angle d'incidence du plan orbital du satellite par rapport
#             a l'equateur
#     alpha : position angulaire du satellite sur son cercle
#     roll : roulie (lateral) appliqu APRES pitch
#     pitch : tangage (avant-arriere) appliqu en premier
#     yaw : lacet (appliqu en dernier)
#     altitude : (normalement fixe). En fait rayon du cercle orbital. 
#                sachant que le rayon de la terre vaut 10.
#     anglePannels : orientation des panneaux solaires
#     lasercolor_r lasercolor_v lasercolor_b : couleurs RGB du laser

package provide Gdhe::proba 1.0

proc proba {angleTerre incid alpha roll pitch yaw altitude anglePannels \
		lasercolor_r lasercolor_v lasercolor_b} {
    pushMatrix

    rotate -$angleTerre 0 0 1
    rotate $incid 1 0 0
    rotate $alpha 0 0 1
    translate $altitude 0 0

    # Axe centre satellite - centre terre
    color 255 255 255
    polyline 2 0 0 0 [ expr $altitude * -3 ] 0 0
    

    # Le cercle de visibilite
    set rVisi 2
    pushMatrix
    translate [ expr 10 - $altitude ] 0 0
    rotate 90 0 1 0
    polygon 12  \
	$rVisi 0  \
	[expr $rVisi * 0.866025] [expr $rVisi * 0.5] \
	[expr $rVisi * 0.5] [expr $rVisi * 0.866025] \
	0 $rVisi \
	[expr $rVisi * -0.5] [expr $rVisi * 0.866025] \
	[expr $rVisi * -0.866025] [expr $rVisi * 0.5] \
	-$rVisi 0 \
	[expr $rVisi * -0.866025] [expr $rVisi * -0.5] \
	[expr $rVisi * -0.5] [expr $rVisi * -0.866025] \
	0 -$rVisi \
	[expr $rVisi * 0.5] [expr $rVisi * -0.866025] \
	[expr $rVisi * 0.866025] [expr $rVisi * -0.5]

    popMatrix

    # Orientation
    rotate $pitch 0 0 1
    rotate $roll 0 1 0
    rotate $yaw 1 0 0

# Les deux petites boules sur rayon laser pour debug
#    pushMatrix
#    translate [ expr 10 - $altitude ] 0 0
#    color 110 200 0
#    sphere 0 0 0 0.1
#    translate 0.5 0 0
#    sphere 0 0 0 0.1
#    popMatrix


    # Le corps cylindrique
    object Proba { 
	color 210 120 120 
	cylinder 0.5 0 0 x 1.0 0.7 -1.0
    }

    # Tir laser
    color $lasercolor_r $lasercolor_v $lasercolor_b 
    polyline 2 -1.5 0 0 [ expr $altitude * -3 ] 0 0
    
    # 
    rotate 90 1 0 0
    rotate $anglePannels 0 1 0
    
    # Les panneaux solaires
    object Pannels {
	# color 110 210 50
	color 150 120 0
	box 0 -1.2 -0.3 0.1 1.1 0.6
	box 0  1.2 -0.3 0.1 1.1 0.6
# ??
#	color 30 89 255
# 	cylinder 0 -2 0 y 0.1 4
    }

    popMatrix
}
